#version 330 core
out vec4 FragColor;

#define NR_POINT_LIGHTS 1

struct Material {
    sampler2D diffuse_tex;
    sampler2D specular_tex;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
};

struct DirLight {
    vec3 direction;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

struct PointLight {
    vec3 position;
    float constant;
    float linear;
    float quadratic;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
    vec2 TexCoords;
    vec4 FragPosLightSpace;
} fs_in;

uniform vec3 viewPos;
uniform vec3 lightPos;

uniform DirLight dirLight;
uniform PointLight pointLights[NR_POINT_LIGHTS];

uniform Material material;
uniform sampler2D shadowMap;
uniform bool shadows; 

// Function prototypes
vec3 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, float shadow);
vec3 CalcPointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir, float shadow);
vec3 SpecularShading(vec3 viewDir, vec3 lightDir, vec3 normal, vec3 specular);
float ShadowCalculation(vec4 fragPosLightSpace);

void main()
{
    // Properties
    vec3 norm = normalize(fs_in.Normal);
    vec3 viewDir = normalize(viewPos - fs_in.FragPos);
    // == ======================================
    // Our lighting is set up in 2 phases: directional and  point lights 
	// In the main() function we take all the calculated colors and sum them up for
    // this fragment's final color.
    // == ======================================
    
	// Calculate shadow
    float shadow = shadows ? ShadowCalculation(fs_in.FragPosLightSpace) : 0.0;                       
    shadow = min(shadow, 0.85); // reduce shadow strength a little: allow some diffuse/specular light in shadowed regions
   
	// Phase 1: Directional lighting
    vec3 result = CalcDirLight(dirLight, norm, viewDir, shadow);
    
	// Phase 2: Point lights
   for(int i = 0; i < NR_POINT_LIGHTS; i++)
        result += CalcPointLight(pointLights[i], norm, fs_in.FragPos, viewDir, shadow);
   float brightness = dot(result, vec3(0.2126, 0.7152, 0.0722)); 
	FragColor = vec4(result, 1.0);
}


// Calculates the color when using a directional light.
vec3 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, float shadow)
{
    vec3 lightDir = normalize(-light.direction);
    // Diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // Combine results
    vec3 ambient = light.ambient *   vec3(texture(material.diffuse_tex, fs_in.TexCoords));
    vec3 diffuse = light.diffuse * diff *   vec3(texture(material.diffuse_tex, fs_in.TexCoords));
    vec3 specular = SpecularShading(viewDir, lightDir, normal, light.specular);
	
    return (ambient + (1.0 - shadow) * (diffuse + specular));
}


// Calculates the color when using a point light.
vec3 CalcPointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir, float shadow)
{
    vec3 lightDir = normalize(light.position - fragPos);
    // Diffuse shading
    float diff = max(dot(normal, lightDir), 0.0);
    // Attenuation
    float distance = length(light.position - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));
    // Combine results
    vec3 ambient = light.ambient * vec3(texture(material.diffuse_tex, fs_in.TexCoords));
    vec3 diffuse = light.diffuse * diff * vec3(texture(material.diffuse_tex, fs_in.TexCoords));
    vec3 specular = SpecularShading(viewDir, lightDir, normal, light.specular);
    return ((ambient + (1.0 - shadow) * (diffuse + specular))*attenuation);
}

vec3 SpecularShading(vec3 viewDir, vec3 lightDir, vec3 normal, vec3 specular)
{
    //vec3 reflectDir = reflect(-lightDir, normal);  
    //float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
	
    vec3 halfwayDir = normalize(lightDir + viewDir);
    float spec = pow(max(dot(normal, halfwayDir), 0.0), material.shininess);
    return  specular * spec * vec3(texture(material.specular_tex, fs_in.TexCoords));
}

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
   
   // Transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    
	float closestDepth = texture(shadowMap, projCoords.xy).r; 
    float currentDepth = projCoords.z;
   
   // Calculate bias (based on depth map resolution and slope)
    vec3 normal = normalize(fs_in.Normal);
    vec3 lightDir = normalize(lightPos - fs_in.FragPos);
    float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005);
        
	// PCF
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
   shadow = (0.5 + (shadow/18.0));
    
    // Keep the shadow at 0.0 when outside the far_plane region of the light's frustum.
    if(projCoords.z > 1.0)
        shadow = 0.0;
        
    return shadow;
}
